define(['dojo/_base/declare',
    'jimu/BaseWidget',
    'dojo/on',
    'dojo/_base/lang',
    'dojo/Deferred',
    'dojo/dom',
    'dijit/form/Select',
    'dijit/form/Button',
    'dijit/layout/ContentPane',
    'esri/tasks/query',
    'esri/tasks/QueryTask',
    'jimu/LayerInfos/LayerInfos',
    'jimu/SelectionManager',
    'esri/graphic',
    "esri/Color",
    'esri/layers/GraphicsLayer',
    'esri/symbols/SimpleFillSymbol',
    'esri/symbols/SimpleLineSymbol',
    'esri/layers/FeatureLayer',
    "esri/request",
    "dojox/charting/Chart",
    "dojox/charting/themes/Wetland",
    'dojo/domReady!'
  ],
  function(declare, BaseWidget, on, lang, Deferred, dom,
    Select, Button, ContentPane,
    Query, QueryTask,
    LayerInfos,
    SelectionManager,
    Graphic,
    Color,
    GraphicsLayer,
    SimpleFillSymbol,
    SimpleLineSymbol,
    FeatureLayer,
    esriRequest,
    Chart, Wetland) {

    return declare([BaseWidget], {
      layer: null,
      fieldX: null,
      fieldY: null,
      url: null,
      type: null,
      lineChart: null,
      barChart: null,
      extensionFilter: null,
      extensionEvent: null,
      selectionManager: SelectionManager.getInstance(),
      testlayer: null,

      startup: function() {
        this.inherited(arguments)
        this.initButton()
      },

      initButton: function() {
        on(this.btnConsulta, 'click', lang.hitch(this, function(){
          this.getWebMapLayer();
        }));
      },

      getWebMapLayer: function() {
        var año1 = this.selectorAño1.value;
        var año2 = this.selectorAño2.value;
        LayerInfos.getInstance(this.map, this.map.itemInfo)
          .then(lang.hitch(this, function(layerInfosObj) {
            var info = layerInfosObj._finalLayerInfos;
            for (i in info) {
              if (info[i].layerObject.url == 'https://portalgis.dotgiscorp.com/dotgis/rest/services/HuellaEVO/FeatureServer/0') {
                this.testlayer = layerInfosObj.getLayerInfoById(info[i].layerObject.id)
                var huella = this.testlayer.layerObject;
                huella.setDefinitionExpression(`año BETWEEN ${año1} and ${año2}`);
                var huella1 = 'https://portalgis.dotgiscorp.com/dotgis/rest/services/HuellaEVO/FeatureServer/0';
                var miQueryTask = new QueryTask(huella1);
                var miQuery = new Query();
                miQuery.where = `año BETWEEN ${año1} and ${año2}`;
                miQuery.orderByFields = ["año ASC"];
                miQuery.outFields = ["*"];
                miQuery.returnGeometry = false;
                miQueryTask.execute(miQuery, lang.hitch(this, this.initCallback), lang.hitch(this, this.initErroback));
              }
            }
          }))
      },

      initCallback: function(result) {
        var arrayResultado = result.features;
        var resultadoAños = [];
        var resultadoPorcen = [];
        var resultadosHec = [];
        var ejes = [];
        var etiquetas = [];
        var areahas = [];

        //Conseguimos los atributos de los años,porcentajes y hectáreas
        for (var i = 0; i < arrayResultado.length; i++) {
          var hue = arrayResultado[i]
          var atributos = hue.attributes;
          var años = atributos.año;
          var porcentaje = atributos.huetmca;
          var hectareas = atributos.crechue;
          var areahac = atributos.areaha;
          areahas.push(areahac);
          resultadoAños.push(años);
          resultadoPorcen.push(porcentaje);
          resultadosHec.push(hectareas);
        }

        //Creamos las etiquetas para nuestro gráfico
        for (var i = 0; i < resultadoAños.length; i++) {
          for (var i = 0; i < resultadoPorcen.length; i++) {
            var obj2 = "" + resultadoAños[i];
            etiquetas.push(obj2);
          }
        };

        //Obtenemos los elementos para el eje x y el eje y
        for (var i = 0; i < resultadoAños.length; i++) {
          for (var i = 0; i < resultadoPorcen.length; i++) {
            var obj = {
              x: resultadoAños[i],
              y: resultadoPorcen[i]
            };
            ejes.push(obj);
          }
        };

        //Sumamnos los porcentajes de crecimiento entre los años seleccionados y lo mostramos como texto
        var porcen = [];
        var suma = 0;
        for (var i = 0; i < resultadoPorcen.length; i++) {
          var resultadoPorcenFix = resultadoPorcen[i].toFixed(2);
          suma = suma + resultadoPorcen[i];
          porcen.push(resultadoPorcenFix)
        }
        var fin = suma;
        var num = fin.toFixed(2);
        var finTexto = num.toString();
        //para comentar el año del principio y el año final en el texto resultante
        var primeraño = etiquetas[0];
        var ultimoaño = etiquetas[etiquetas.length - 1];
        var txtPorcen = this.txtCiudades;
        txtPorcen.innerHTML = finTexto + " " + "%" + " " + "de" + " " + "crecimiento" + " " + "entre" + " " + primeraño + " " + "y" + " " + ultimoaño;

        //Se crea el gráfico
        var popCanvas = this.popChart;

        if (window.bar != undefined){
          window.bar.destroy();
        }
        window.bar = new Chart(popCanvas, {
          type: 'line',
          data: {
            labels: etiquetas,
            datasets: [{
              label: 'Porcentaje de crecimiento',
              data: ejes,
              backgroundColor: [
                '#FF7F50'
              ]
            }]
          }
        });

        var table = '<table><thead><tr><th>Año</th><th>Tamaño huella (ha)</th><th>Crecimiento (%)</th><th>Crecimiento interanual (ha)</th></tr></thead><tbody>';

        for (var i = 0; i < resultadoAños.length; i++) {
          table += '<tr><td>' + resultadoAños[i] + '</td><td>' + areahas[i] + '</td><td>' + porcen[i] + '<td>' + resultadosHec[i] + '</td></tr>';
        }
        table += '</tbody></table>';
        this.text.innerHTML = table;
      },

      initErroback: function miErrback() {
        alert("Ha habido un problema , vuelve a ejecutar la consulta.");
      }
    });
  });
